// $Id: clamav-sockwrite.c,v 1.2 2003/08/15 23:12:58 ensc Exp $    --*- c++ -*--

// Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; version 2 of the License.
//  
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//  
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//  

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <unistd.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdlib.h>
#include <stdio.h>
#include <errno.h>
#include <string.h>

#ifndef UNIX_PATH_MAX
#  define UNIX_PATH_MAX	108
#endif

#define WRITEMSG(FD,X)	(void)write(FD, X, sizeof(X)-1)

static size_t
xread(int fd, void *buf, size_t len)
{
  ssize_t	res;
  do {
    res = read(fd, buf, len);
  } while (res==-1 && (errno==EINTR || errno==EAGAIN));

  if (res==-1) {
    perror("read()");
    exit(1);
  }

  return (size_t)(res);
}

static void
xwrite(int fd, void const *buf, size_t len)
{
  char const *	ptr = buf;
  while (len>0) {
    ssize_t	tmp;
    do {
      tmp = write(fd, ptr, len);
    } while (tmp==-1 && (errno==EINTR || errno==EAGAIN));

    if (tmp==-1) {
      perror("write()");
      exit(1);
    }

    if ((size_t)tmp>len) {
      WRITEMSG(2, "Internal error\n");
      exit(1);
    }

    ptr += tmp;
    len -= tmp;
  }
}

int main(int argc, char *argv[])
{
  int			sock;
  struct sockaddr_un	addr;
  
  if (argc!=2) {
    WRITEMSG(2, "Usage: clamav-sockwrite FILENAME\n");
    return EXIT_FAILURE;
  }

  sock = socket(AF_UNIX, SOCK_STREAM, 0);
  if (sock==-1) {
    perror("socket()");
    return EXIT_FAILURE;
  }

  memset(&addr, 0, sizeof addr);
  addr.sun_family = AF_UNIX;
  strncpy(addr.sun_path, argv[1], sizeof(addr.sun_path));
  addr.sun_path[UNIX_PATH_MAX-1]='\0';

  if (connect(sock, (void const *)&addr, sizeof addr)==-1) {
    perror("socket()");
    return EXIT_FAILURE;
  }

  for (;;) {
    char	buf[512];
    
    size_t	len = xread(0, buf, sizeof buf);
    if (len==0) break;
    xwrite(sock, buf, len);
  }

  for (;;) {
    char	buf[512];
    size_t	len = xread(sock, buf, sizeof buf);
    if (len==0) break;

    xwrite(1, buf, len);
  }
    
  if (close(sock)==-1) {
    perror("close()");
    return EXIT_FAILURE;
  }

  return EXIT_SUCCESS;
}


  /// Local Variables:
  /// compile-command: "diet gcc clamav-sockwrite.c -W -Wall -pedantic -std=gnu89 -D_GNU_SOURCE"
  /// End:
